Package: compSPOT
Type: Package
Title: compSPOT: Tool for identifying and comparing significantly
        mutated genomic hotspots
Version: 1.10.0
Authors@R: c(
    person("Sydney", "Grant", email = "Sydney.Grant@roswellpark.org", 
    role = c("aut", "cre"), 
    comment = c(ORCID = "0000-0003-1849-5921")),
    person("Ella", "Sampson", email = "ellasamp@buffalo.edu", role = c("aut")),
    person("Rhea", "Rodrigues", 
    email = "RheaCarmelGlen.Rodrigues@roswellpark.org", 
    role = c("aut"), comment = c(ORCID = "0000-0002-8573-8658")),
    person("Gyorgy", "Paragh", email = "Gyorgy.Paragh@roswellpark.org", 
    role = c("aut"), comment = c(ORCID = "0000-0002-6612-9267")))
Description: Clonal cell groups share common mutations within cancer,
        precancer, and even clinically normal appearing tissues. The
        frequency and location of these mutations may predict prognosis
        and cancer risk. It has also been well established that certain
        genomic regions have increased sensitivity to acquiring
        mutations. Mutation-sensitive genomic regions may therefore
        serve as markers for predicting cancer risk. This package
        contains multiple functions to establish significantly mutated
        hotspots, compare hotspot mutation burden between samples, and
        perform exploratory data analysis of the correlation between
        hotspot mutation burden and personal risk factors for cancer,
        such as age, gender, and history of carcinogen exposure. This
        package allows users to identify robust genomic markers to help
        establish cancer risk.
License: Artistic-2.0
Encoding: UTF-8
LazyData: FALSE
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.2.3
biocViews: Software, Technology, Sequencing, DNASeq, WholeGenome,
        Classification, SingleCell, Survival, MultipleComparison
Imports: stats, base, ggplot2, plotly, magrittr, ggpubr, gridExtra,
        utils, data.table
Suggests: BiocStyle, knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
URL: https://github.com/sydney-grant/compSPOT
BugReports: https://github.com/sydney-grant/compSPOT/issues
Config/testthat/edition: 3
Depends: R (>= 4.3.0)
Config/pak/sysreqs: cmake make libicu-dev libuv1-dev libssl-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 13:01:21 UTC
RemoteUrl: https://github.com/bioc/compSPOT
RemoteRef: RELEASE_3_23
RemoteSha: 1d761dbe40e29bfcd0d01a002f033e7608696c9e
NeedsCompilation: no
Packaged: 2026-05-03 16:10:12 UTC; root
Author: Sydney Grant [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1849-5921>),
  Ella Sampson [aut],
  Rhea Rodrigues [aut] (ORCID: <https://orcid.org/0000-0002-8573-8658>),
  Gyorgy Paragh [aut] (ORCID: <https://orcid.org/0000-0002-6612-9267>)
Maintainer: Sydney Grant <Sydney.Grant@roswellpark.org>
Built: R 4.6.0; ; 2026-05-03 16:45:20 UTC; unix

# Generated by roxygen2: do not edit by hand

export(compare_features)
export(compare_groups)
export(find_hotspots)
importFrom(data.table,rbindlist)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotate)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_dotplot)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,geom_violin)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,position_dodge)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,stat_ecdf)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggpubr,ggarrange)
importFrom(ggpubr,stat_compare_means)
importFrom(ggpubr,stat_cor)
importFrom(gridExtra,grid.arrange)
importFrom(magrittr,"%>%")
importFrom(plotly,add_trace)
importFrom(plotly,ggplotly)
importFrom(plotly,layout)
importFrom(plotly,plot_ly)
importFrom(stats,ks.test)
importFrom(stats,wilcox.test)
importFrom(utils,combn)

#' plot all features based on hotspot mutation burden
#'
#' @description
#' Creates a scatter plot of numerical features vs hotspot mutation burden and
#' calculates the pearson correlation.
#' Creates a violin plot for categorical features and calculates p-value based
#' on either Wilcox test or Kruskal-Wallis test.
#'
#'
#' @param i a numerical value based on which feature in feature list to pull
#' from
#' @param df a dataframe containing the hotspot mutation count, and values for
#' each feature for all samples in dataset
#' @param feature A list containing all the features.
#'
#' @return ggplot objects of comparison of feature vs hotspot mutation burden in all
#' samples
#'
#' @keywords internal
#'
#' @examples
#'
#' plot_features(1, df = sample_df, feature = feature)
#'
#' @noRd
#'

plot_features <- function(i, df, feature) {
  df <- na.omit(df)
  feature_df <- df[, feature]
  if (typeof(feature_df[, i]) %in% c("numeric", "integer", "double")) {
    plot <-
      ggplot2::ggplot(data = df, aes(x = feature_df[, i], y = Count)) +
      ggplot2::geom_point(color = "brown4", fill = "brown4") +
      ggpubr::stat_cor() +
      ggplot2::theme_classic() +
      ggplot2::xlab(feature[[i]]) +
      ggplot2::theme(
        axis.text.x = element_text(
          angle = 45,
          vjust = 0.5,
          hjust = 0.5,
          size = 12
        ),
        axis.title.y = element_text(size = 12),
        axis.title.x = element_text(size = 12),
        axis.title = element_text(size = 4),
        axis.text.y = element_text(size = 12),
        plot.title = element_text(hjust = 0.5, size = 12)
      )
  }
  else{
    plot <-
      ggplot2::ggplot(df,
                      aes(
                        x = feature_df[, i],
                        y = Count,
                        group = feature_df[, i],
                        fill = feature_df[, i]
                      )) +
      ggplot2::geom_violin(trim = FALSE) +
      ggpubr::stat_compare_means() +
      ggplot2::scale_fill_manual(values = c(
        "brown4",
                "darksalmon",
                "tan",
                "darkorange",
                "white",
                "grey"
      )) +
      ggplot2::scale_color_manual(values = c(
        "brown4",
                "darksalmon",
                "tan",
                "darkorange",
                "white",
                "grey"
      )) +
      ggplot2::theme_classic() +
      ggplot2::xlab(feature[[i]]) +
      ggplot2::theme(
        axis.text.x = element_text(
          angle = 45,
          vjust = 1,
          hjust = 1,
          size = 12
        ),
        axis.title.y = element_text(size = 12),
        axis.title.x = element_text(size = 12),
        axis.title = element_text(size = 4),
        axis.text.y = element_text(size = 12),
        plot.title = element_text(hjust = 0.5, size = 12),
        legend.position = "none"
      )
  }
  return(plot)
}






#' hotspot comparison by additional features
#'
#' @description
#' This function performs an exploratory data analysis
#' comparing the relationship between user-input features to
#' hotspot mutation burden.
#' @details
#' This function is used to classify the features into
#' sequential features if values are numerical or classifies
#' them into #' categorical features. Sequential features are
#' compared to the mutation count using Pearson correlation.
#' Similarly, in categorical features either Wilcox or
#' Kruskal-Wallis test is used to compare between the groups
#' in the features based on the mutational count. Scatter plot
#' is used to represent the sequential features along with the
#' R and p-value from the pearson correlation. Violin plots are
#' used to plot the groups in the categorical data and Wilcox
#' or Kruskal-Wallis values are shown on the graph.
#'
#'
#' @param data A dataframe containing the clinical features and
#' the mutation count. Dataframe must contain columns with the following names:
#' "Chromosome" <-- Chromosome number where the mutation is located
#' "Position" <-- Genomic position number where the mutation is located
#' "Sample" <-- Unique ID for each sample in dataset
#' "Gene" <-- Name of the gene which mutation is located in (optional)

#' @param feature A list containing all the features.
#' @param regions a dataframe containing the chromosome, start
#' and end base pair position of each region of interest
#' @importFrom ggplot2 ggplot aes theme theme_classic
#' scale_color_manual element_text geom_point geom_smooth xlab annotate
#' ylab geom_violin scale_fill_manual
#' @importFrom ggpubr stat_cor stat_compare_means ggarrange
#' @importFrom utils combn
#' @importFrom stats wilcox.test
#' @importFrom gridExtra grid.arrange
#'
#'
#'
#' @return A grid of all the violin plots for the categorical
#' data and scatter plot for the sequential data.
#'
#'
#' @examples
#'
#' data("compSPOT_example_mutations")
#' data("compSPOT_example_regions")
#' features <- c("AGE", "SEX", "ADJUVANT_TX", "SMOKING_HISTORY",
#' "TUMOR_VOLUME", "KI_67")
#' compare_features(data = compSPOT_example_mutations,
#' regions = compSPOT_example_regions, feature = features)
#'
#' @export
#'

compare_features <- function(data, regions, feature) {
  if (base::isFALSE(is.data.frame(data))) {
    stop("Input data must be in the form of data.frame")
  }
  if (base::isFALSE(is.data.frame(regions))) {
    stop("Input regions must be in the form of data.frame")
  }
  if (base::isFALSE("Chromosome" %in% colnames(data))) {
    stop("input data must contain column 'Chromosome'")
  }
  if (base::isFALSE("Position" %in% colnames(data))) {
    stop("input data must contain column 'Position'")
  }
  if (base::isFALSE("Sample" %in% colnames(data))) {
    stop("input data must contain column 'Sample'")
  }
  if (base::isFALSE("Chromosome" %in% colnames(regions))) {
    stop("input regions must contain column 'Chromosome'")
  }
  if (base::isFALSE("Lowerbound" %in% colnames(regions))) {
    stop("input regions must contain column 'Lowerbound'")
  }
  if (base::isFALSE("Upperbound" %in% colnames(regions))) {
    stop("input regions must contain column 'Upperbound'")
  }
  if (base::isFALSE(is.numeric(data$Position))) {
    stop("column 'Position' must be in the form of numeric")
  }
  if (base::isFALSE(is.numeric(regions$Lowerbound))) {
    stop("column 'Lowerbound' must be in the form of numeric")
  }
  if (base::isFALSE(is.numeric(regions$Upperbound))) {
    stop("column 'Upperbound' must be in the form of numeric")
  }
  count_mutations_features <- function(x, regions, data) {
    sub <- subset(data, Sample == x)
    count <- 0
    for (i in seq_len(nrow(regions))) {
      sub <-
        subset(
          sub,
          Chromosome == regions$Chromosome[[i]] &
            Position %in% regions$Lowerbound[[i]]:regions$Upperbound[[i]]
        )
      count <- count + nrow(sub)
    }
    return(count)
  }
  count.ls <-
    lapply(
      unique(data$Sample),
      count_mutations_features,
      regions = regions,
      data = data
    )
  count.df <-
    data.frame("Sample" = unique(data$Sample),
               "Count" = unlist(count.ls))
  clin.df <- data[, c("Sample", feature)]
  clin.df <- unique(clin.df)
  sample_df <- merge(clin.df, count.df, by = "Sample")
  plot_ls <-
    lapply(seq_along(feature),
           plot_features,
           df = sample_df,
           feature = feature)
  plots <-
    ggpubr::ggarrange(plotlist = plot_ls,
                      nrow = 2,
                      ncol = ceiling(length(plot_ls) / 2))
  return(plots)
}

#' compSPOT example genomic regions
#' @name compSPOT_example_regions
#' @docType data
#' @aliases compSPOT_example_regions
#' @title Genomic Coordinates of Regions of Interest
#' @usage compSPOT_example_regions
#' @keywords datasets
#'
#' @return A dataframe containing the chromosome number, base pair location,
#' and gene names of 200 genomic regions
#' highly mutated in Lung Squamous Cell Carcinoma identified using seq.hotSPOT
#'
#' @format ## `example_regions`
#' a dataframe with 2 columns and 200 rows:
#' \describe{
#'  \item{Lowerbound}{Base pair position of the start of the region}
#'  \item{Upperbound}{Base pair position of the end of the region}
#'  \item{Chromosome}{Chromosome which the mutation is located on}
#'  \item{Gene}{Name of gene affected by mutation}
#'
#' }
#'
#' @description A dataframe containing the chromosome number, lowerbound and
#' upperbound base pair locations of each region of
#' interest along with the name of the gene where the region is located. Each
#' row indicates a unique region. Regions were
#' identified using the seq.hotSPOT package based on Lung Squamous Cell
#' Carcinoma highly mutated regions.
#'
#' @references
#' Grant SR et al;
#' HotSPOT: A Computational Tool to Design Targeted Sequencing Panels to Assess
#' Early Photocarcinogenesis.
#' Cancers (Basel). 2023 Mar 5;15(5):1612. doi: 10.3390/cancers15051612.
#' PMID: 36900402; PMCID: PMC10001346.
#'
#' Grant S, Wei L, Paragh G (2023). seq.hotSPOT: Targeted sequencing panel
#' design based on mutation hotspots.
#' R package version 1.0.0, https://github.com/sydney-grant/seq.hotSPOT.
#'
NULL

#' count hotspot mutations in each group
#'
#' @description
#' Based on sample name from data, count number of mutations
#' in hotspot of interest based on each group
#'
#'
#' @param sample_name a character of sample name of interest
#' @param data a dataframe containing the chromosome, base pair
#' position, and optionally gene name of each mutation.
#' @param regions a dataframe containing the chromosome, start
#' and end base pair position of each region of interest
#' @param spot an integer containing the row number of hotspot
#' from regions dataframe
#' @param include_genes true or false whether gene names are
#' included in regions dataframe
#'
#' @return A dataframe with the hotspot, group, and mutation count from input
#' sample name
#'
#' @keywords internal
#'
#' @examples
#'
#' count <- count_mutations(sample_name = "CRUK0062-R1", data = data,
#' regions = regions, spot = spot, include_genes = TRUE)
#'
#' @noRd
#'
count_mutations <-
  function(sample_name,
           data,
           regions,
           spot,
           include_genes) {
    sub <- subset(data, Sample == sample_name)
    count <-
      nrow(
        subset(
          sub,
          Chromosome == regions$Chromosome[[spot]] &
            Position %in%
            regions$Lowerbound[[spot]]:regions$Upperbound[[spot]]
        )
      )
    if (include_genes == TRUE) {
      spot_data <- data.frame(
        "Hotspot" = paste(
          regions$Gene[[spot]],
          " ",
          regions$Chromosome[[spot]],
          ":",
          regions$Lowerbound[[spot]],
          "-",
          regions$Upperbound[[spot]],
          sep = ""
        ),
        "Group" = unique(sub$Group),
        "Mutation_Count" = count
      )
    }
    if (include_genes == FALSE) {
      spot_data <-
        data.frame(
          "Hotspot" = paste(
            regions$Chromosome[[spot]],
            ":",
            regions$Lowerbound[[spot]],
            "-",
            regions$Upperbound[[spot]],
            sep = ""
          ),
          "Group" = unique(sub$Group),
          "Mutation_Count" = count
        )
    }
    return(spot_data)
  }

#' find differences in mutatino burden per hotspot in different groups
#'
#' @description
#' Based on sample name from data, count number of mutations
#' in hotspot of interest based on each group
#'
#' @param data a dataframe containing the chromosome, base pair
#' position, and optionally gene name of each mutation.
#' @param return a value of either "plot" or "hotspots" based on which type of
#' data will be returned
#' @param regions a dataframe containing the chromosome, start
#' and end base pair position of each region of interest
#' @param spot an integer containing the row number of hotspot
#' from regions dataframe
#' @param include_genes true or false whether gene names are
#' included in regions dataframe
#' @param pvalue a threshold p-value for Kolmogorov-Smirnov test
#' @param threshold the cutoff empirical distribution for
#' Kolmogorov-Smirnov test
#'
#' @return A dataframe with either the mutation count for each group in each
#' hotspots, or a dataframe of the distance and p-value of hotspots with
#' significant differences between the two groups.
#'
#' @keywords internal
#'
#' @examples
#'
#' group <- count_groups(spot = 1, regions = regions, name1 = "High-Risk",
#' name2 = "Low-Risk", include_genes = TRUE, return = "plot")
#'
#' @noRd
#'
count_groups <-
  function(spot,
           data,
           regions,
           name1,
           name2,
           include_genes,
           return,
           pvalue,
           threshold) {
    plot_data <- data.table::rbindlist(
      lapply(
        unique(data$Sample),
        count_mutations,
        data = data,
        regions = regions,
        spot = spot,
        include_genes =
          include_genes
      )
    )
    count_1 <- subset(plot_data, Group == name1)$Mutation_Count
    count_2 <- subset(plot_data, Group == name2)$Mutation_Count
    if (sum(c(count_1, count_2)) > 0) {
      ks <- ks.test(count_1, count_2, alternative = "two.sided")
      if (ks$p.value < pvalue & ks$statistic >= threshold) {
        if (sum(count_1) > sum(count_2)) {
          high_group <- "Group 1"
        }
        else{
          high_group <- "Group 2"
        }
        if (include_genes == TRUE) {
          diff_hotspots <- data.frame(
            "D" = ks$statistic,
            "pval" = ks$p.value,
            "Higher Group" = high_group,
            "Hotspot" = paste(
              regions$Gene[[spot]],
              " ",
              regions$Chromosome[[spot]],
              ":",
              regions$Lowerbound[[spot]],
              "-",
              regions$Upperbound[[spot]],
              sep = ""
            )
          )
        }
        if (include_genes == FALSE) {
          diff_hotspots <-
            data.frame(
              "D" = ks$statistic,
              "pval" = ks$p.value,
              "Higher Group" = high_group,
              "Hotspot" = paste(
                regions$Chromosome[[spot]],
                ":",
                regions$Lowerbound[[spot]],
                "-",
                regions$Upperbound[[spot]],
                sep = ""
              )
            )
        }
      }
      if (ks$p.value >= pvalue |
          ks$statistic < threshold) {
        diff_hotspots <- data.frame()
      }
    }
    if (sum(c(count_1, count_2)) == 0) {
      diff_hotspots <- data.frame()
    }
    if (return == "plot") {
      return(plot_data)
    }
    if (return == "hotspots") {
      return(diff_hotspots)
    }
  }


#' hotspot group comparison with plot
#'
#' @description
#' Based on a dataframe of hotspots, this function groups the
#' hotspots into two different groups and overlays interactive
#' violin plots based on mutation count for comparison.
#'
#'
#' @importFrom ggplot2 ggplot aes stat_ecdf theme theme_classic
#' scale_color_manual element_text geom_point geom_vline xlab ylab
#' geom_violin scale_fill_manual geom_smooth position_dodge geom_dotplot
#' @importFrom plotly ggplotly plot_ly add_trace layout
#' @importFrom magrittr %>%
#' @importFrom gridExtra grid.arrange
#' @param plot_data a dataframe containing the hotspot,
#' mutation count, and associated groups
#' @param group1 a string containing the name of one group for
#' the comparison
#' @param group2 a string containing the name of the second group
#' for the comparison
#'
#' @return A plotly object violin plot displaying the comparison of
#' mutation counts for each group.
#'
#' @keywords internal
#'
#' @examples
#'
#' data("plot_data")
#' fig <- plot_spot(plot_data = plot_data, group1 = "High-Risk",
#' group2 = "Low-Risk")
#'
#' @noRd


plot_spot <- function(plot_data, group1, group2) {
  plot1 <-
    ggplot2::ggplot(data = plot_data,
                    ggplot2::aes(y = Mutation_Count,
                                 x = Hotspot,
                                 fill = Group)) +
    ggplot2::geom_violin(alpha = 0.75,
                         trim = FALSE,
                         position = position_dodge()) +
    ggplot2::scale_fill_manual(values = c("brown4", "darksalmon")) +
    ggplot2::scale_color_manual(values = c("brown4", "darksalmon")) +
    ggplot2::theme_classic() +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(
        angle = 45,
        vjust = 0.5,
        hjust = 0.5,
        size = 10
      ),
      axis.title.y = ggplot2::element_text(size = 12),
      axis.title.x = ggplot2::element_text(size = 12),
      axis.title = ggplot2::element_text(size = 4),
      axis.text.y = ggplot2::element_text(size = 12),
      plot.title = ggplot2::element_text(hjust = 0.5, size = 12)
    )
  plot1 <- plotly::ggplotly(plot1)
  return(plot1)
}




#' ECDF plot of hotspot mutation comparison
#'
#' @description
#' Based on a dataframe of hotspots, this function creates a ECDF plot to
#' compare the difference in hotspot mutation burden between two groups.
#'
#'
#' @importFrom ggplot2 ggplot aes stat_ecdf theme theme_classic
#' scale_color_manual element_text geom_point geom_vline xlab ylab
#' geom_violin scale_fill_manual geom_smooth
#' @param spot a string containing the name of a unique hotspot in present in
#' plot_data
#' @param plot_data a dataframe containing the hotspot,
#' mutation count, and associated groups
#'
#' @return A ggplot object ECDF plot displaying the comparison of
#' mutation counts for each group.
#'
#' @keywords internal
#'
#' @examples
#'
#' data("plot_data")
#' fig <- plot_ecdf(spot = "TP53 17:7578429-7578528",
#' plot_data = plot_data,
#' group1 = "High-Risk",
#' group2 = "Low-Risk")
#'
#' @noRd


plot_ecdf <- function(spot, plot_data) {
  plot_data.sub <- subset(plot_data, Hotspot == spot)
  plot2 <- ggplot2::ggplot(data = plot_data.sub,
                           ggplot2::aes(x = Mutation_Count, group = Group,
                                        col = Group)) +
    ggplot2::stat_ecdf(geom = "step") +
    ggplot2::theme_classic() +
    ggplot2::scale_color_manual(values = c("brown4", "darksalmon")) +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(
        angle = 90,
        vjust = 0.5,
        hjust = 0.5,
        size = 10
      ),
      axis.title.y = ggplot2::element_text(size = 12),
      axis.title.x = ggplot2::element_text(size = 12),
      axis.title = ggplot2::element_text(size = 4),
      axis.text.y = ggplot2::element_text(size = 12),
      plot.title = ggplot2::element_text(hjust = 0.5, size = 12)
    )
  return(plot2)
}

#' hotspot comparison by group
#'
#' @description
#' This function compares the mutation frequency of a panel of
#' genomic regions between two sub-groups.
#'
#'
#' @details
#' This function creates a list of mutation frequency per unique
#' sample for each genomic regions separated based on specified
#' sub-groups. The regions with significant differences in mutation
#' distribution are calculated using a Kolmogorov-Smirnov test. The
#' difference in mutation frequency is output in a violin plot.
#'
#' @importFrom stats ks.test
#' @importFrom magrittr %>%
#' @importFrom data.table rbindlist
#' @param data a dataframe containing the chromosome, base pair
#' position, and optionally gene name of each mutation. Dataframe must contain
#' columns with the following names:
#' "Chromosome" <-- Chromosome number where the mutation is located
#' "Position" <-- Genomic position number where the mutation is located
#' "Sample" <-- Unique ID for each sample in dataset
#' "Gene" <-- Name of the gene which mutation is located in (optional)
#' "Group" <-- Group classification ID (group.spot only)
#' @param regions a dataframe containing the chromosome, start and
#' end base pair position of each region of interest
#' @param pvalue a threshold p-value for Kolmogorov-Smirnov test
#' @param threshold the cutoff empirical distribution for
#' Kolmogorov-Smirnov test
#' @param name1 a string containing the name of one group for the
#' comparison
#' @param name2 a string containing the name of the second group for
#' the comparison
#' @param include_genes true or false whether gene names are included
#' in regions dataframe
#'
#' @return a list containing the following:
#' 1. A dataframe with the hotspot, group, and mutation count from input
#' sample name
#' 2. A plotly object violin plot comparing the mutation frequency per sample in groups as
#' given by "name1" and "name2" variables
#' 3. An array of ECDF plots comparing the mutation frequency per sample in
#' groups as given by "name1" and "name2" variables
#'
#' @examples
#'
#' data("compSPOT_example_mutations")
#' data("compSPOT_example_regions")
#' compare_groups(data = compSPOT_example_mutations,
#' regions = compSPOT_example_regions, pvalue = 0.05, threshold = 0.4,
#' name1 = "High-Risk", name2 = "Low-Risk", include_genes = TRUE)
#'
#' @export
#'


compare_groups <-
  function(data,
           regions,
           pvalue,
           threshold,
           name1,
           name2,
           include_genes) {
    if (base::isFALSE(is.data.frame(data))) {
      stop("Input data must be in the form of data.frame")
    }
    if (base::isFALSE(is.data.frame(regions))) {
      stop("Input regions must be in the form of data.frame")
    }
    if (base::isFALSE("Chromosome" %in% colnames(data))) {
      stop("input data must contain column 'Chromosome'")
    }
    if (base::isFALSE("Position" %in% colnames(data))) {
      stop("input data must contain column 'Position'")
    }
    if (base::isFALSE("Sample" %in% colnames(data))) {
      stop("input data must contain column 'Sample'")
    }
    if (base::isFALSE("Group" %in% colnames(data))) {
      stop("input data must contain column 'Group'")
    }
    if (base::isFALSE(length(unique(data$Group)) == 2)) {
      stop("input data must have two unique group labels")
    }
    if (base::isFALSE("Chromosome" %in% colnames(regions))) {
      stop("input regions must contain column 'Chromosome'")
    }
    if (base::isFALSE("Lowerbound" %in% colnames(regions))) {
      stop("input regions must contain column 'Lowerbound'")
    }
    if (base::isFALSE("Upperbound" %in% colnames(regions))) {
      stop("input regions must contain column 'Upperbound'")
    }
    if (base::isFALSE(is.numeric(data$Position))) {
      stop("column 'Position' must be in the form of numeric")
    }
    if (base::isFALSE(is.numeric(regions$Lowerbound))) {
      stop("column 'Lowerbound' must be in the form of numeric")
    }
    if (base::isFALSE(is.numeric(regions$Upperbound))) {
      stop("column 'Upperbound' must be in the form of numeric")
    }
    if (base::isTRUE(include_genes) &
        base::isFALSE("Gene" %in% colnames(data)))
    {
      stop("input data must contain column 'Gene'")
    }
    if (base::isTRUE(include_genes) &
        base::isFALSE(is.character(data$Gene))) {
      stop("column 'Gene' must be in the form of character")
    }
    if (pvalue > 1) {
      stop("pvalue cannot be greater than 1")
    }
    if (threshold > 1) {
      stop("threshold cannot be greater than 1")
    }
    if (pvalue < 0) {
      stop("pvalue cannot be less than 0")
    }
    if (threshold < 0) {
      stop("threshold cannot be less than 0")
    }
    plot_data <-
      data.table::rbindlist(
        lapply(
          seq_len(nrow(regions)),
          count_groups,
          data = data,
          regions = regions,
          name1 = "High-Risk",
          name2 = "Low-Risk",
          include_genes = include_genes,
          return = "plot",
          pvalue = pvalue,
          threshold = threshold
        )
      )
    all_diff <- data.table::rbindlist(
      lapply(
        seq_len(nrow(regions)),
        count_groups,
        data = data,
        regions = regions,
        name1 = "High-Risk",
        name2 = "Low-Risk",
        include_genes = include_genes,
        return = "hotspots",
        pvalue = pvalue,
        threshold = threshold
      )
    )
    plot2_list <- lapply(unique(plot_data$Hotspot), plot_ecdf,
                         plot_data = plot_data)
    plot <- plot_spot(plot_data, group1 = name1,
                      group2 = name2)
    return_items <-
      list(all_diff,
           plot,
           ggpubr::ggarrange(
             plotlist = plot2_list,
             nrow = 2,
             ncol = ceiling(length(plot2_list)
                            / 2)
           ))
    return(return_items)
  }

data("compSPOT_example_mutations")
data("compSPOT_example_regions")


test_that("check number significant spots", {
    significant_spots <-
        find_hotspots(
            data = compSPOT_example_mutations,
            regions = compSPOT_example_regions,
            pvalue = 0.05,
            threshold = 0.2,
            include_genes = TRUE,
            rank = TRUE
        )
    hotspots <- subset(significant_spots[[1]], type == "Hotspot")
    comp_test <- compare_groups(
        data = compSPOT_example_mutations,
        regions = hotspots,
        pvalue = 0.05,
        threshold = 0.4,
        name1 = "High-Risk",
        name2 = "Low-Risk",
        include_genes = TRUE
    )
    expect_true(nrow(comp_test[[1]]) <= length(unique(hotspots$Label)))
})

#' genomic region ranker
#'
#' @description
#' ranks all genomic regions from highest to lowest
#' mutation frequency in given dataset
#'
#' @param data A dataframe containing the location of
#' each mutation.
#' @param regions a dataframe containing the location
#' of gene region of interest
#' @param include_genes true or false whether gene names
#' are included in regions dataframe
#'
#' @keywords internal
#'
#' @return A dataframe containing the ranked genomic
#' regions from highest to lowest mutation frequency
#'
#'
#' @examples
#'
#' data("mutation_data")
#' spot_rank(data = mutation_data, regions = ea_regions,
#' include_genes = TRUE)
#'
#'@noRd

spot_rank <- function(i, data, regions, include_genes) {
  data_sub <- subset(
    data,
    Chromosome == regions$Chromosome[[i]] &
      Position %in% regions$Lowerbound[[i]]:regions$Upperbound[[i]]
  )
  if (include_genes == TRUE) {
    ranked_row <-
      data.frame(
        "Chromosome" = regions$Chromosome[[i]],
        "Lowerbound" = regions$Lowerbound[[i]],
        "Upperbound" = regions$Upperbound[[i]],
        "Count" = nrow(data_sub),
        "Gene" = regions$Gene[[i]]
      )
  }
  if (include_genes == FALSE) {
    ranked_row <-
      data.frame(
        "Chromosome" = regions$Chromosome[[i]],
        "Lowerbound" = regions$Lowerbound[[i]],
        "Upperbound" = regions$Upperbound[[i]],
        "Count" = nrow(data_sub)
      )
  }
  return(ranked_row)
}

#' create labels for final hotspot dataframe
#'
#' @description
#' creates a unique label for each hotspot based on the gene name and genomic
#' location.
#'
#' @param regions a dataframe containing the location
#' of gene region of interest
#' @param include_genes true or false whether gene names
#' are included in regions dataframe
#' #' @param i a numeric value of which row from the region dataframe to make a
#' label for
#'
#' @keywords internal
#'
#' @return A string containing the unique label for hotspot
#'
#'
#' @examples
#'
#' create_labels(i = 1, regions = compSPOT_example_regions,
#' include_genes = TRUE)
#'
#'@noRd
#'


create_labels <- function(i, regions, include_genes) {
  if (base::isTRUE(include_genes)) {
    label <- paste(
      regions$Gene[[i]],
      " ",
      regions$Chromosome[[i]],
      ":",
      regions$Lowerbound[[i]],
      "-",
      regions$Upperbound[[i]],
      sep = ""
    )
  }
  if (base::isFALSE(include_genes)) {
    label <- paste(regions$Chromosome[[i]],
                   ":",
                   regions$Lowerbound[[i]],
                   "-",
                   regions$Upperbound[[i]],
                   sep = "")
  }
  return(label)
}

#' counting mutations per regions
#'
#' @description
#' creates a unique label for each hotspot based on the gene name and genomic
#' location.
#'
#' @param reg_sub subset dataframe of total regions dataframe which contains
#' either top or bottom genomic regions for comparison
#' @param s a string containing ID for unique sample in mutation dataset
#' @param data a dataframe containing the chromosome, base pair position,
#' and optionally gene name of each mutation
#' @keywords internal
#'
#' @return A list of mutation frequencies per genomic region.
#'
#'
#' @examples
#'
#' region_counter(s = "CRUK0062-R1", reg_sub = reg1)
#'
#'@noRd
#'

region_counter <- function(s, reg_sub, data) {
  reg_list <- c()
  df_sub <- subset(data, Sample == s)
  reg_count <- 0
  for (m in seq_len(nrow(reg_sub))) {
    reg_count <-
      reg_count + nrow(
        subset(
          df_sub,
          Chromosome == reg_sub$Chromosome[[m]]
          &
            Position %in% reg_sub$Upperbound[[m]]:reg_sub$Lowerbound[[m]]
        )
      )
  }
  reg_list <- c(reg_list, (reg_count / nrow(reg_sub)))
  return(reg_list)
}





#' sig.spot visualization
#'
#' @description
#' Based on a panel of genomic regions, this function visualizes the
#' regions which are found to have significance based on percentage
#' of samples with hotspot mutations.
#'
#' @details
#' This function begins by displaying a plot with a ranked number of
#' hotspot based on number of mutations on the x-axis versus percentage
#' of all samples with mutations in hotspot on the y-axis. Regions of high
#' significance are ranked on the left side of the plot. Based on the
#' type, the points plotted as red dots are the hotspots which were found
#' to have a significant number of mutations whereas the pink were
#' found to not be significant. An interactive label of each point is also
#' available. A second plot is also created using the same data used in the
#' Kolmogorov-Smirnov calculation to show a visualization of the ECDF
#' of hotspots vs non-hotspots.
#'
#' @importFrom ggplot2 ggplot aes stat_ecdf theme theme_classic
#' scale_color_manual element_text geom_point geom_vline xlab ylab
#' @importFrom plotly ggplotly
#' @param df1 a dataframe containing the labeled region number,
#' type of region, percentage of samples with
#' hotspot mutation
#' @param df2 a dataframe containing the data required for calculation
#' of ECDF
#'
#' @return A list of two plotly objects, one organizing the region number in ranked order versus the
#' percentage of the sample with hotspot mutation which also denotes the
#' label of the data point is displayed when hovering over the item, and a second
#' plotly object providing a visualization of the ECDF for hotspot vs non-hotspots.
#'
#' @keywords internal
#' @examples
#'
#' data("regions")
#' data("ecdf_df")
#' plot <- plot_sigspot(df1 = region, df2 = ecdf_df)
#'
#'@noRd


plot_sigspot <- function(df1, df2) {
  plot1 <-
    ggplot2::ggplot(df1, ggplot2::aes(
      y = percent,
      x = number,
      text = paste("Label", Label)
    )) +
    ggplot2::geom_point(
      data = subset(df1, type == "Hotspot"),
      color = "brown4",
      fill = "brown4",
      alpha = 1,
      shape = 21,
      size = 1.5
    ) +
    ggplot2::geom_point(
      data = subset(df1, type == "Non-hotspot"),
      color = "darksalmon",
      fill = "darksalmon",
      alpha = 1,
      shape = 21,
      size = 1.5
    ) +
    ggplot2::geom_vline(
      xintercept = (nrow(subset(
        df1, type == "Hotspot"
      )) + 1),
      linetype = "longdash",
      linewidth = 0.5
    ) +
    ggplot2::theme_classic() +
    ggplot2::ylab("Percentage of Samples \n with Hotspot Mutation") +
    ggplot2::xlab("Ranked Region Number") +
    ggplot2::theme(
      axis.text.x = element_text(
        angle = 45,
        vjust = 0.5,
        hjust = 0.5,
        size = 12
      ),
      axis.title.y = element_text(size = 12),
      axis.title.x = element_text(size = 12),
      axis.title = element_text(size = 4),
      axis.text.y = element_text(size = 12),
      plot.title = element_text(hjust = 0.5, size = 12)
    )
  plot1 <- plotly::ggplotly(plot1, tooltip =  c("text"))
  plot2 <-
    ggplot2::ggplot(data = df2, ggplot2::aes(x = Count, group = Group,
                                             col = Group)) +
    ggplot2::stat_ecdf(geom = "step") +
    ggplot2::theme_classic() +
    ggplot2::scale_color_manual(values = c("brown4", "darksalmon"))
  ggplot2::theme(
    axis.text.x = ggplot2::element_text(
      angle = 45,
      vjust = 0.5,
      hjust = 0.5,
      size = 12
    ),
    axis.title.y = ggplot2::element_text(size = 12),
    axis.title.x = ggplot2::element_text(size = 12),
    axis.title = ggplot2::element_text(size = 4),
    axis.text.y = ggplot2::element_text(size = 12),
    plot.title = ggplot2::element_text(hjust = 0.5, size = 12)
  )
  plot2 <- plotly::ggplotly(plot2, tooltip =  c("text"))
  output <- list(plot1, plot2)
  return(output)
}


#' significant hotspot calculator
#'
#' @description
#' Based on a panel of genomic regions, this function calculates the
#' regions which are found to have significantly higher mutation
#' frequency compared to less mutated regions.
#'
#' @details
#' This function begins by measuring the mutation frequency for each
#' unique sample for each provided genomic region. Beginning with the
#' top ranked hotspot, a Kolmogorov-Smirnov test is preformed on the
#' mutation frequency of the top genomic region compared to the normalized
#' mutation frequency of all the lower-ranked regions. This continues,
#' then running the Kolmogorov-Smirnov test for the normalized mutation
#' frequency of the top 2 genomic regions compared to the normalized
#' mutation frequency of all lower-ranked regions.This process repeats itself,
#' continuously adding an additional genomic regions each time until either
#' the set p-value or empirical distribution threshold is not met. Once this
#' cutoff has been reached, an established list of mutation hotspots is
#' provided.
#'
#' @importFrom stats ks.test
#' @param data a dataframe containing the chromosome, base pair
#' position, and optionally gene name of each mutation. Dataframe must contain
#' columns with the following names:
#' "Chromosome" <-- Chromosome number where the mutation is located
#' "Position" <-- Genomic position number where the mutation is located
#' "Sample" <-- Unique ID for each sample in dataset
#' "Gene" <-- Name of the gene which mutation is located in (optional)
#' @param regions a dataframe containing the chromosome, start and end
#' base pair position of each region of interest
#' @param pvalue the p-value cutoff for included hotspots
#' @param threshold the cutoff empirical distribution for Kolmogorov-Smirnov
#' test
#' @param include_genes true or false whether gene names are included in
#' regions dataframe
#' @param rank true or false whether regions dataframe is already ranked and
#' includes mutation count of total dataset
#'
#' @return A list containing the following:
#' 1. dataframe containing the genomic regions with significant
#' mutation frequency
#' 2. plotly object Dotplot showing the percentage of samples with mutations in each ranked
#' genomic region, highlighting significantly mutated hotspots
#' 3. plotly object ECDF plot showing the difference in mutation frequency between hotspots
#' and non-hotspots
#'
#'
#' @examples
#'
#' data("compSPOT_example_mutations")
#' data("compSPOT_example_regions")
#' significant_spots <- find_hotspots(data = compSPOT_example_mutations,
#' regions = compSPOT_example_regions,
#' pvalue = 0.05, threshold = 0.2, include_genes = TRUE, rank = TRUE)
#'
#'
#' @export
#'


find_hotspots <-
  function(data,
           regions,
           pvalue,
           threshold,
           include_genes,
           rank) {
    if (base::isFALSE(is.data.frame(data))) {
      stop("Input data must be in the form of data.frame")
    }
    if (base::isFALSE(is.data.frame(regions))) {
      stop("Input regions must be in the form of data.frame")
    }
    if (base::isFALSE("Chromosome" %in% colnames(data))) {
      stop("input data must contain column 'Chromosome'")
    }
    if (base::isFALSE("Position" %in% colnames(data))) {
      stop("input data must contain column 'Position'")
    }
    if (base::isFALSE("Sample" %in% colnames(data))) {
      stop("input data must contain column 'Sample'")
    }
    if (base::isFALSE("Chromosome" %in% colnames(regions))) {
      stop("input regions must contain column 'Chromosome'")
    }
    if (base::isFALSE("Lowerbound" %in% colnames(regions))) {
      stop("input regions must contain column 'Lowerbound'")
    }
    if (base::isFALSE("Upperbound" %in% colnames(regions))) {
      stop("input regions must contain column 'Upperbound'")
    }
    if (base::isFALSE(is.numeric(data$Position))) {
      stop("column 'Position' must be in the form of numeric")
    }
    if (base::isFALSE(is.numeric(regions$Lowerbound))) {
      stop("column 'Lowerbound' must be in the form of numeric")
    }
    if (base::isFALSE(is.numeric(regions$Upperbound))) {
      stop("column 'Upperbound' must be in the form of numeric")
    }
    if (base::isTRUE(include_genes) &
        base::isFALSE("Gene" %in% colnames(data)))
    {
      stop("input data must contain column 'Gene'")
    }
    if (base::isTRUE(include_genes) &
        base::isFALSE(is.character(data$Gene))) {
      stop("column 'Gene' must be in the form of character")
    }
    if (pvalue > 1) {
      stop("pvalue cannot be greater than 1")
    }
    if (threshold > 1) {
      stop("threshold cannot be greater than 1")
    }
    if (pvalue < 0) {
      stop("pvalue cannot be less than 0")
    }
    if (threshold < 0) {
      stop("threshold cannot be less than 0")
    }
    if (base::isFALSE(rank) &
        base::isFALSE("Count" %in% colnames(data))) {
      stop("input data must contain column 'Count', or set rank = TRUE")
    }
    if (base::isTRUE(rank)) {
      regions <-
        data.table::rbindlist(
          lapply(
            seq_len(nrow(regions)),
            spot_rank,
            data = data,
            regions = regions,
            include_genes = include_genes
          )
        )
      regions <- regions[order(-regions$Count), ]
    }
    pv <- pvalue - 1
    d <- threshold + 1
    i <- 1
    while (pv < pvalue & d > threshold) {
      reg1 <- regions[seq_len(i), ]
      reg2 <- regions[(i + 1):nrow(regions), ]
      reg1_list <- unlist(lapply(
        unique(data$Sample),
        region_counter,
        reg_sub = reg1,
        data = data
      ))
      reg2_list <- unlist(lapply(
        unique(data$Sample),
        region_counter,
        reg_sub = reg2,
        data = data
      ))
      ks <- ks.test(reg1_list, reg2_list, alternative = "greater")
      pv <- ks$p.value
      d <- ks$statistic
      i <- i + 1
    }
    sig_spots <- i - 2
    ecdf_df <- data.frame("Count" = c(reg1_list, reg2_list),
                          "Group" = c(rep("Hotspot", length(reg1_list)),
                                      rep("Non-hotspot", length(reg1_list))))
    regions$number <- seq_len(nrow(regions))
    regions$percent <-
      regions$Count / length(unique(data$Sample)) * 100
    perc_cutoff <- regions$percent[[sig_spots]]
    regions$type <-
      c(rep("Hotspot", nrow(subset(
        regions, percent >= perc_cutoff
      ))),
      rep("Non-hotspot", nrow(subset(
        regions, percent < perc_cutoff
      ))))
    
    labels <-
      lapply(seq_len(nrow(regions)),
             create_labels,
             regions = regions,
             include_genes = include_genes)
    regions$Label <- unlist(labels)
    plots <- plot_sigspot(regions, ecdf_df)
    return_list <- list(regions, plots[[1]], plots[[2]])
    return(return_list)
  }

data("compSPOT_example_mutations")
data("compSPOT_example_regions")

pval <- 0.05
thres <- 0.2

test_that("spots are valid", {
  spots <-
    find_hotspots(
      data = compSPOT_example_mutations,
      regions = compSPOT_example_regions,
      pvalue = pval,
      threshold = thres,
      include_genes = TRUE,
      rank = TRUE
    )
  hotspots <- subset(spots[[1]], type == "Hotspot")
  nonhotspots <- subset(spots[[1]], type == "Non-hotspot")
  cutoff <- hotspots$percent[[nrow(hotspots)]]
  if (max(nonhotspots$percent) < min(hotspots$percent)) {
    check = TRUE
  }
  else{
    check = FALSE
  }
  
  expect_true(check)
})

#' compSPOT example mutation data
#' @name compSPOT_example_mutations
#' @docType data
#' @aliases compSPOT_example_mutations
#' @title Single Nucleotide Variants and Patient Features in Lung Cancer
#' Patients
#' @usage compSPOT_example_mutations
#' @keywords datasets
#'
#' @return A dataframe containing the chromosome number, base pair location,
#' sample ID, gene name, patient features including:
#' age, sex, adjuvant therapy treatment, smoking history, tumor volume, ki67
#' quantification, and risk-classification for
#' cancer progression.
#'
#' @format ## `example_mutations`
#' a dataframe with 11 columns and 22947 rows:
#' \describe{
#'  \item{Sample}{ID assigned to indicate each unique sample}
#'  \item{Gene}{Name of gene affected by mutation}
#'  \item{Chromosome}{Chromosome which the mutation is located on}
#'  \item{Position}{Base pair position of mutation}
#'  \item{AGE}{Age of the patient}
#'  \item{SEX}{Sex of the patient}
#'  \item{ADJUVANT_TX}{Statement of whether or not patient recieved adjuvant
#'  therapy}
#'  \item{SMOKING_HISTORY}{Patient's history of smoking}
#'  \item{TUMOR_VOLUME}{Measured volumne of patient's lung tumor}
#'  \item{KI_67}{Quantification of ki67 markers observed in each patient}
#'  \item{Group}{Risk classification of patients based on observed survival}
#' }
#'
#' @description A dataframe containing the chromosome number, base pair
#' location, sample ID, gene name,
#' patient features including: age, sex, adjuvant therapy treatment, smoking
#' history, tumor volume,
#' ki67 quantification, and risk-classification for cancer progression. Data
#' curated from cBioPortal dataset:
#' Non-Small Cell Lung Cancer (TRACERx, NEJM & Nature 2017)
#'
#' @references
#' Abbosh C et al.;
#' TRACERx consortium; PEACE consortium;
#' Swanton C. Phylogenetic ctDNA analysis depicts early-stage lung cancer
#' evolution.
#' Nature. 2017 Apr 26;545(7655):446-451. doi: 10.1038/nature22364. Erratum in:
#' Nature. 2017 Dec 20;:
#' PMID: 28445469; PMCID: PMC5812436.
#'
#' Jamal-Hanjani M et al.;
#' TRACERx Consortium. Tracking the Evolution of Non-Small-Cell Lung Cancer.
#' N Engl J Med. 2017 Jun 1;376(22):2109-2121. doi: 10.1056/NEJMoa1616288.
#' Epub 2017 Apr 26. PMID: 28445112.
#'
#'
NULL

feature <- c("AGE", "SEX", "SMOKING_HISTORY")

data("compSPOT_example_mutations")
data("compSPOT_example_regions")

count_mutations_features <- function(x, regions, data) {
  sub <- subset(data, Sample == x)
  count <- 0
  for (i in seq_len(nrow(regions))) {
    sub <-
      subset(
        sub,
        Chromosome == regions$Chromosome[[i]] &
          Position %in% regions$Lowerbound[[i]]:regions$Upperbound[[i]]
      )
    count <- count + nrow(sub)
  }
  return(count)
}




test_that("correct number of plots", {
  count.ls <-
    lapply(
      unique(compSPOT_example_mutations$Sample),
      count_mutations_features,
      regions = compSPOT_example_regions,
      data = compSPOT_example_mutations
    )
  count.df <-
    data.frame(
      "Sample" = unique(compSPOT_example_mutations$Sample),
      "Count" = unlist(count.ls)
    )
  clin.df <- compSPOT_example_mutations[, c("Sample", feature)]
  clin.df <- unique(clin.df)
  sample_df <- merge(clin.df, count.df, by = "Sample")
  
  plot_ls <-
    lapply(seq_along(feature),
           plot_features,
           df = sample_df,
           feature = feature)
  expect_equal(length(feature), length(plot_ls))
})

data("compSPOT_example_mutations")
data("compSPOT_example_regions")

shuffled_regions <-
  compSPOT_example_regions[sample(1:nrow(compSPOT_example_regions)),]

regions <-
  data.table::rbindlist(
    lapply(
      seq_len(nrow(shuffled_regions)),
      spot_rank,
      data = compSPOT_example_mutations,
      regions = shuffled_regions,
      include_genes = TRUE
    )
  )
ranked_regions <- regions[order(-regions$Count), ]


comp_values <- function(i, reg){
  v1 <- reg$Count[[i]]
  v2 <- reg$Count[[i + 1]]
  test_that("correct ranking", {
    expect_gte(v1, v2)
  })
}

lapply(1:(nrow(ranked_regions)-1),
       comp_values, reg = ranked_regions)

---
title: "compSPOT-Vignette"
author:
- name: Sydney Grant
  affiliation: Roswell Park Comprehensive Cancer Center
  email: sydney.grant@roswellpark.org
- name: Ella Sampson
  affiliation: Roswell Park Comprehensive Cancer Center
  email: ellasamp@buffalo.edu
- name: Rhea Rodrigues
  affiliation: Roswell Park Comprehensive Cancer Center
  email: RheaCarmelGlen.Rodrigues@roswellpark.org
- name: Gyorgy Paragh
  affiliation: Roswell Park Comprehensive Cancer Center
  email: Gyorgy.Paragh@roswellpark.org
package: compSPOT
output:
  BiocStyle::html_document:
  toc: true
  theme: cerulean
vignette: |
  %\VignetteIndexEntry{compSPOT-Vignette}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```



# Introduction

Sydney R. Grant^1,2^, Ella Sampson^1^, Rhea Rodrigues^1,2^, Gyorgy Paragh^1,2^

^1^Department of Dermatology, Roswell Park Comprehensive Cancer Center, Buffalo, NY
^2^Department of Cell Stress Biology, Roswell Park Comprehensive Cancer Center, Buffalo, NY

Clonal cell groups share common mutations within cancer, precancer, and even 
clinically normal appearing tissues. The frequency and location of these 
mutations may predict prognosis and cancer risk. It has also been well 
established that certain genomic regions have increased sensitivity to acquiring 
mutations. Mutation-sensitive genomic regions may therefore serve as markers 
for predicting cancer risk. This package contains multiple functions to 
establish significantly mutated hotspots, compare hotspot mutation burden 
between samples, and perform exploratory data analysis of the correlation 
between hotspot mutation burden and personal risk factors for cancer, such as 
age, gender, and history of carcinogen exposure. This package allows users to 
identify robust genomic markers to help establish cancer risk.

Currently, minimal resources exist which enable researchers to design their own 
targeted sequencing panels based on specific biological questions and tissues 
of interest. `compSPOT` has been designed to work sequentially with Bioconductor 
package `seq.hotSPOT`. Highly mutated genomic regions identified by `seq.hotSPOT` 
may be used for discovery of significant mutation hotspots with `compSPOT`. 
`compSPOT` may also be used to discover differences in hotspot mutation burden 
between different groups of interest, and the association of mutation burden with 
clinical features. `compSPOT` may be used in combination with the Bioconductor 
package `RTCGA.mutations`, which can be used to pull mutation datasets from the 
TCGA database to be used as input data in various cancer types. Additionally, 
the package `RTCGA.clinical` may be also used to identify highly mutated regions 
in subsets of patients with specific clinical features of interest.

# Installation & Setup

```{r, eval = FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("compSPOT")
```

Load [*compSPOT*][]

```{r}
library(compSPOT)
```

# Example Input Data

## Mutation Data

The mutation dataset should include the following columns:
"Chromosome" <-- Chromosome number where the mutation is located
"Position" <-- Genomic position number where the mutation is located
"Sample" <-- Unique ID for each sample in dataset
"Gene" <-- Name of the gene which mutation is located in (optional)
"Group" <-- Group classification ID (for compare_groups only) 
Clinical Parameters <-- (for compare_features only)

Loading example mutations:
``` {r load mutations}
data("compSPOT_example_mutations")
```


## Regions Data

The regions dataset should include the following columns:
"Chromosome" <-- Chromosome number where the region is located
"Lowerbound" <-- Genomic position number where the region begins
"Upperbound" <-- Genomic position number where the region ends
"Gene" <-- Name of the gene which mutation is located in (optional)
"Count" <-- Number of mutations in mutation dataset which are found within the 
region (optional)

Loading example regions:
``` {r load regions}
data("compSPOT_example_regions")
```

# Example Workflow

The compSPOT package contains three main functions for (1) selection of 
mutation hotspots (2) comparison of hotspot mutation burden between groups, 
and (3) comparison of mutation hotspot enrichment based on clinical and personal 
risk factors. All functions return both numerical outputs based on analysis 
summary and data visualization components for quick and easy interpretation of 
results.

## Identifying Mutation Hotspots with find_hotspots

Our previously published Bioconductor package `seq.hotSPOT` 
(doi: 10.3390/cancers15051612) identifies highly mutated genomic regions based 
on SNV datasets. While this tool can identify long lists of mutated regions, 
we sought to establish a method for identifying which of these genomic regions 
have significantly higher mutation frequency compared to others and may be used 
as markers of carcinogenic progression.


Methods: This function begins by measuring the mutation frequency for each 
unique sample for each provided genomic region. Beginning with the top-ranked 
hotspot, a Kolmogorov-Smirnov test is performed on the mutation frequency of 
the top genomic region compared to the normalized mutation frequency of all the 
lower-ranked regions. This continues, then running the Kolmogorov-Smirnov test 
for the normalized mutation frequency of the top 2 genomic regions compared to 
the normalized mutation frequency of all lower-ranked regions. This process 
repeats itself, continuously adding an additional genomic regions each time 
until either the set p-value or empirical distribution threshold is not met. 
Once this cutoff has been reached, an established list of mutation hotspots is 
provided.

```{r sig.spots}
significant_spots <- find_hotspots(data = compSPOT_example_mutations, 
                                   regions = compSPOT_example_regions, 
                                   pvalue = 0.05, threshold = 0.2, 
                                   include_genes = TRUE, 
                                   rank = TRUE)
```

Table 1. Example output table from find_hotspots function. 
This table is stored in the first position of the output list.

```{r table 1}
head(significant_spots[[1]])
```


## Comparison Mutation Hotspot Burden with compare_groups

Previously, we have shown mutation hotspots identified using seq.hotSPOT may be 
used to differentiate between samples with history of frequent vs infrequent 
carcinogen exposure (doi: 10.3390/cancers15051612, doi: 10.3390/ijms24097852). 
compare_groups provides an automated approach for statistical and visual 
comparison between mutation enrichment of different groups of interest.


Methods: This function creates a list of mutation frequency per unique sample 
for each genomic region separated based on specified sub-groups. The regions 
with significant differences in mutation distribution are calculated using a 
Kolmogorov-Smirnov test. The difference in mutation frequency is output in a 
violin plot.

For this example dataset, the sig.spot function identified 6 hotspots. We will 
use these 6 hotspots to compared the mutation burden between Lung Cancer 
patients with high- and low-risk of disease progression.


```{r group.spot}
hotspots <- subset(significant_spots[[1]], type == "Hotspot")

group_comp <- compare_groups(data = compSPOT_example_mutations, 
                             regions = hotspots, pval = 0.05, 
                             threshold = 0.2, 
                             name1 = "High-Risk", 
                             name2 = "Low-Risk", 
                             include_genes = TRUE)
```

Table 2. Example output table from compare_groups function.
This table is stored in the first position of the output list.

```{r table 2}
group_comp[[1]]
```


## EDA of Mutation Hotspot Burden and Personal Risk Factors with compare_features 

Mutation enrichment in cancer mutation hotspots has been shown to relate to 
personal cancer risk factors such as age, gender, and carcinogen exposure 
history and may be used in combination to create predictive models of cancer 
risk (doi: 10.3390/ijms24097852). feature.spot provides a baseline analysis of 
any set of clinical features to identify trends in the enrichment of mutations 
and personal risk factors.

Methods: This function first classifies the features into sequential or 
categorical features. Sequential features are compared to the mutation count 
using Pearson Correlation. Similarly, in categorical features Wilcox Rank Sum 
and Kruska-Wallis Tests are used to compare groups within the features based on 
their mutational count. 


```{r feature.spot}
features <- c("AGE", "SEX", "SMOKING_HISTORY", "TUMOR_VOLUME", "KI_67")
feature_example <- compare_features(data = compSPOT_example_mutations, 
                                    regions = compSPOT_example_regions, 
                                    feature = features)
```



```{r sessionInfo, echo=FALSE}
sessionInfo()
```

\name{compSPOT}
\alias{compSPOT}

\title{
Mutation hotspot finder and comparison of hotspot burden.
}
\description{
It is well known that numerous clones of cells sharing common
mutations exist within cancer, precancer, and even clinically
normal appearing tissues. The frequency and location of
these mutations may aid in the prediction of cancer risk of
certain individuals. It has also been well established that
certain genomic regions have increased sensitivity to acquiring
mutations. Mutation-sensitive genomic regions may therefore be
used as markers for prediction of cancer risk. This package
contains multiple functions for the establishment of significantly
mutated hotspots, comparison of hotspot mutation burden between
sub-groups, and exploratory data analysis of the correlation between
hotspot mutation burden, and personal risk factors for cancer such
as age, gender, and history of carcinogen exposure. This package
aims to allow users to identify robust genomic markers
which may serve as markers of cancer risk.
}
\value{
A package containing functions which find statistically significant
mutation hotspots and compare mutation hotspot burden between groups
and correlation between clinical features.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare_groups.R
\name{compare_groups}
\alias{compare_groups}
\title{hotspot comparison by group}
\usage{
compare_groups(data, regions, pvalue, threshold, name1, name2, include_genes)
}
\arguments{
\item{data}{a dataframe containing the chromosome, base pair
position, and optionally gene name of each mutation. Dataframe must contain
columns with the following names:
"Chromosome" <-- Chromosome number where the mutation is located
"Position" <-- Genomic position number where the mutation is located
"Sample" <-- Unique ID for each sample in dataset
"Gene" <-- Name of the gene which mutation is located in (optional)
"Group" <-- Group classification ID (group.spot only)}

\item{regions}{a dataframe containing the chromosome, start and
end base pair position of each region of interest}

\item{pvalue}{a threshold p-value for Kolmogorov-Smirnov test}

\item{threshold}{the cutoff empirical distribution for
Kolmogorov-Smirnov test}

\item{name1}{a string containing the name of one group for the
comparison}

\item{name2}{a string containing the name of the second group for
the comparison}

\item{include_genes}{true or false whether gene names are included
in regions dataframe}
}
\value{
a list containing the following:
\enumerate{
\item A dataframe with the hotspot, group, and mutation count from input
sample name
\item A plotly object violin plot comparing the mutation frequency per sample in groups as
given by "name1" and "name2" variables
\item An array of ECDF plots comparing the mutation frequency per sample in
groups as given by "name1" and "name2" variables
}
}
\description{
This function compares the mutation frequency of a panel of
genomic regions between two sub-groups.
}
\details{
This function creates a list of mutation frequency per unique
sample for each genomic regions separated based on specified
sub-groups. The regions with significant differences in mutation
distribution are calculated using a Kolmogorov-Smirnov test. The
difference in mutation frequency is output in a violin plot.
}
\examples{

data("compSPOT_example_mutations")
data("compSPOT_example_regions")
compare_groups(data = compSPOT_example_mutations,
regions = compSPOT_example_regions, pvalue = 0.05, threshold = 0.4,
name1 = "High-Risk", name2 = "Low-Risk", include_genes = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compSPOT_example_regions.R
\docType{data}
\name{compSPOT_example_regions}
\alias{compSPOT_example_regions}
\title{Genomic Coordinates of Regions of Interest}
\format{
\subsection{\code{example_regions}}{

a dataframe with 2 columns and 200 rows:
\describe{
\item{Lowerbound}{Base pair position of the start of the region}
\item{Upperbound}{Base pair position of the end of the region}
\item{Chromosome}{Chromosome which the mutation is located on}
\item{Gene}{Name of gene affected by mutation}

}
}
}
\usage{
compSPOT_example_regions
}
\value{
A dataframe containing the chromosome number, base pair location,
and gene names of 200 genomic regions
highly mutated in Lung Squamous Cell Carcinoma identified using seq.hotSPOT
}
\description{
A dataframe containing the chromosome number, lowerbound and
upperbound base pair locations of each region of
interest along with the name of the gene where the region is located. Each
row indicates a unique region. Regions were
identified using the seq.hotSPOT package based on Lung Squamous Cell
Carcinoma highly mutated regions.
}
\details{
compSPOT example genomic regions
}
\references{
Grant SR et al;
HotSPOT: A Computational Tool to Design Targeted Sequencing Panels to Assess
Early Photocarcinogenesis.
Cancers (Basel). 2023 Mar 5;15(5):1612. doi: 10.3390/cancers15051612.
PMID: 36900402; PMCID: PMC10001346.

Grant S, Wei L, Paragh G (2023). seq.hotSPOT: Targeted sequencing panel
design based on mutation hotspots.
R package version 1.0.0, https://github.com/sydney-grant/seq.hotSPOT.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare_features.R
\name{compare_features}
\alias{compare_features}
\title{hotspot comparison by additional features}
\usage{
compare_features(data, regions, feature)
}
\arguments{
\item{data}{A dataframe containing the clinical features and
the mutation count. Dataframe must contain columns with the following names:
"Chromosome" <-- Chromosome number where the mutation is located
"Position" <-- Genomic position number where the mutation is located
"Sample" <-- Unique ID for each sample in dataset
"Gene" <-- Name of the gene which mutation is located in (optional)}

\item{regions}{a dataframe containing the chromosome, start
and end base pair position of each region of interest}

\item{feature}{A list containing all the features.}
}
\value{
A grid of all the violin plots for the categorical
data and scatter plot for the sequential data.
}
\description{
This function performs an exploratory data analysis
comparing the relationship between user-input features to
hotspot mutation burden.
}
\details{
This function is used to classify the features into
sequential features if values are numerical or classifies
them into #' categorical features. Sequential features are
compared to the mutation count using Pearson correlation.
Similarly, in categorical features either Wilcox or
Kruskal-Wallis test is used to compare between the groups
in the features based on the mutational count. Scatter plot
is used to represent the sequential features along with the
R and p-value from the pearson correlation. Violin plots are
used to plot the groups in the categorical data and Wilcox
or Kruskal-Wallis values are shown on the graph.
}
\examples{

data("compSPOT_example_mutations")
data("compSPOT_example_regions")
features <- c("AGE", "SEX", "ADJUVANT_TX", "SMOKING_HISTORY",
"TUMOR_VOLUME", "KI_67")
compare_features(data = compSPOT_example_mutations,
regions = compSPOT_example_regions, feature = features)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compSPOT_example_mutation.R
\docType{data}
\name{compSPOT_example_mutations}
\alias{compSPOT_example_mutations}
\title{Single Nucleotide Variants and Patient Features in Lung Cancer
Patients}
\format{
\subsection{\code{example_mutations}}{

a dataframe with 11 columns and 22947 rows:
\describe{
\item{Sample}{ID assigned to indicate each unique sample}
\item{Gene}{Name of gene affected by mutation}
\item{Chromosome}{Chromosome which the mutation is located on}
\item{Position}{Base pair position of mutation}
\item{AGE}{Age of the patient}
\item{SEX}{Sex of the patient}
\item{ADJUVANT_TX}{Statement of whether or not patient recieved adjuvant
therapy}
\item{SMOKING_HISTORY}{Patient's history of smoking}
\item{TUMOR_VOLUME}{Measured volumne of patient's lung tumor}
\item{KI_67}{Quantification of ki67 markers observed in each patient}
\item{Group}{Risk classification of patients based on observed survival}
}
}
}
\usage{
compSPOT_example_mutations
}
\value{
A dataframe containing the chromosome number, base pair location,
sample ID, gene name, patient features including:
age, sex, adjuvant therapy treatment, smoking history, tumor volume, ki67
quantification, and risk-classification for
cancer progression.
}
\description{
A dataframe containing the chromosome number, base pair
location, sample ID, gene name,
patient features including: age, sex, adjuvant therapy treatment, smoking
history, tumor volume,
ki67 quantification, and risk-classification for cancer progression. Data
curated from cBioPortal dataset:
Non-Small Cell Lung Cancer (TRACERx, NEJM & Nature 2017)
}
\details{
compSPOT example mutation data
}
\references{
Abbosh C et al.;
TRACERx consortium; PEACE consortium;
Swanton C. Phylogenetic ctDNA analysis depicts early-stage lung cancer
evolution.
Nature. 2017 Apr 26;545(7655):446-451. doi: 10.1038/nature22364. Erratum in:
Nature. 2017 Dec 20;:
PMID: 28445469; PMCID: PMC5812436.

Jamal-Hanjani M et al.;
TRACERx Consortium. Tracking the Evolution of Non-Small-Cell Lung Cancer.
N Engl J Med. 2017 Jun 1;376(22):2109-2121. doi: 10.1056/NEJMoa1616288.
Epub 2017 Apr 26. PMID: 28445112.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_hotspots.R
\name{find_hotspots}
\alias{find_hotspots}
\title{significant hotspot calculator}
\usage{
find_hotspots(data, regions, pvalue, threshold, include_genes, rank)
}
\arguments{
\item{data}{a dataframe containing the chromosome, base pair
position, and optionally gene name of each mutation. Dataframe must contain
columns with the following names:
"Chromosome" <-- Chromosome number where the mutation is located
"Position" <-- Genomic position number where the mutation is located
"Sample" <-- Unique ID for each sample in dataset
"Gene" <-- Name of the gene which mutation is located in (optional)}

\item{regions}{a dataframe containing the chromosome, start and end
base pair position of each region of interest}

\item{pvalue}{the p-value cutoff for included hotspots}

\item{threshold}{the cutoff empirical distribution for Kolmogorov-Smirnov
test}

\item{include_genes}{true or false whether gene names are included in
regions dataframe}

\item{rank}{true or false whether regions dataframe is already ranked and
includes mutation count of total dataset}
}
\value{
A list containing the following:
\enumerate{
\item dataframe containing the genomic regions with significant
mutation frequency
\item plotly object Dotplot showing the percentage of samples with mutations in each ranked
genomic region, highlighting significantly mutated hotspots
\item plotly object ECDF plot showing the difference in mutation frequency between hotspots
and non-hotspots
}
}
\description{
Based on a panel of genomic regions, this function calculates the
regions which are found to have significantly higher mutation
frequency compared to less mutated regions.
}
\details{
This function begins by measuring the mutation frequency for each
unique sample for each provided genomic region. Beginning with the
top ranked hotspot, a Kolmogorov-Smirnov test is preformed on the
mutation frequency of the top genomic region compared to the normalized
mutation frequency of all the lower-ranked regions. This continues,
then running the Kolmogorov-Smirnov test for the normalized mutation
frequency of the top 2 genomic regions compared to the normalized
mutation frequency of all lower-ranked regions.This process repeats itself,
continuously adding an additional genomic regions each time until either
the set p-value or empirical distribution threshold is not met. Once this
cutoff has been reached, an established list of mutation hotspots is
provided.
}
\examples{

data("compSPOT_example_mutations")
data("compSPOT_example_regions")
significant_spots <- find_hotspots(data = compSPOT_example_mutations,
regions = compSPOT_example_regions,
pvalue = 0.05, threshold = 0.2, include_genes = TRUE, rank = TRUE)


}
